/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Clippings.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2007-2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


//
// Clippings version 3.0 initialization library
//
// Dependencies:
//  - utils.js
//  - clippak.js


/*
 * Constructor
 */
function aeClippings3(aClippingsSvc, aStringBundle) 
{
  if (! aClippingsSvc) {
    throw this.E_CTOR_MISSING_ARG;
  }

  this._clippingsSvc = aClippingsSvc;
  this._appUtils = new aeClippingsUtils();
  this._strBundle = aStringBundle;
}

aeClippings3.prototype = {
  _clippingsSvc:  null,
  _appUtils:      null,
  _strBundle:     null,
  E_CTOR_MISSING_ARG:  "nsIClippingsService argument to aeClippings3 constructor is missing or undefined",
  E_CLIPPINGSSVC_NOT_INITIALIZED: "nsIClippingsService not initialized"
};


/*
 * Starts the initialization process.
 * Returns true if the entire initialization process completed, or false if
 * initialization was aborted.  Note that just because this method may return
 * true doesn't necessarily mean that initialization succeeded!
 */
aeClippings3.prototype.startInit = function ()
{
  this._initCommonClippingsPref();
  try {
    this._initDataSource();
  }
  catch (e) {
    return false;
  }

  this._importPackagedClippings();
  this._processEmptyFolders();

  return true;
};


/*
 * Initializes the datasource.  Common datasource pref should be set before
 * invoking this method.
 * Throws an exception if datasource initialization failed.
 */
aeClippings3.prototype._initDataSource = function ()
{
  var path = this._appUtils.getDataSourcePathURL();
  var ds = this._clippingsSvc.getDataSource(path + this._appUtils.CLIPDAT_FILE_NAME);
  var backupDirURL = this._appUtils.getDataSourcePathURL() + this._appUtils.BACKUP_DIR_NAME;

  this._clippingsSvc.setBackupDir(backupDirURL);
};


/*
 * Import clippings from the packaged data source file, if it exists.
 * Prerequisite: Datasource must be already initialized.
 */
aeClippings3.prototype._importPackagedClippings = function ()
{
  var packagedClippings = new aePackagedClippings();
  if (packagedClippings.exists()) {
    this._appUtils.log("It appears that a packaged clipping datasource file exists");

    try {
      packagedClippings.import(this._clippingsSvc);
    }
    catch (e if e == packagedClippings.E_FLUSH_FAILED) {
      this._appUtils.alertEx(this._strBundle.getString("appName"),
			     this._strBundle.getString("errorFlushAfterPDSImport"));
    }
    catch (e) {
      this._appUtils.alertEx(this._strBundle.getString("appName"),
			     this._strBundle.getString("errorDSImportFailure"));
      var consoleSvc = Components.classes["@mozilla.org/consoleservice;1"]
	                         .getService(Components.interfaces
					               .nsIConsoleService);
      var msg = String.ae_fmt("Packaged Clipping import error: %s", e);
      consoleSvc.logStringMessage(msg);
    }
  }
};


/*
 * Initializes common clippings pref setting
 */
aeClippings3.prototype._initCommonClippingsPref = function ()
{
  if (this._appUtils.PORTABLE_APP_BUILD) {
    // Disable common clippings if installing into Portable Firefox or
    // Portable Thunderbird
    this._appUtils.setPref("clippings.datasource.common", false);    
    this._appUtils.log("aeClippings3._initCommonClippingsPref(): This Clippings build is designed for use on Portable Firefox or Portable Thunderbird.\nCommon datasource disabled -- datasource located in user profile directory");
    return;
  }

  var clippings1x, clippings2, commonClippings, prefValue;
  var oldClipdatFile, clipdat2File, clipdat2CommonFile;  // nsIFile objects
  oldClipdatFile = this._appUtils.getCurrentProfileDir();
  clipdat2File = this._appUtils.getCurrentProfileDir();
  clipdat2File.append(this._appUtils.CLIPDAT_FILE_NAME);
  oldClipdatFile.append(this._appUtils.CLIPDAT_1X_FILE_NAME);
  clipdat2CommonFile = this._appUtils.getHomeDir();
  clipdat2CommonFile.append(this._appUtils.CLIPDAT_FILE_NAME);
 
  if (oldClipdatFile.exists() && oldClipdatFile.isFile()) {
    this._appUtils.log("aeClippings3._initCommonClippingsPref(): Detected Clippings 1.x datasource in user profile directory");
    clippings1x = true;
  }

  if (clipdat2File.exists() && clipdat2File.isFile()) {
    this._appUtils.log("aeClippings3._initCommonClippingsPref(): Detected Clippings 2.x/3.x datasource in user profile directory");
    clippings2 = true;
  }

  if (clipdat2CommonFile.exists() && clipdat2CommonFile.isFile()) {
    commonClippings = true;
    this._appUtils.log("aeClippings3._initCommonClippingsPref(): Common datasource detected");
  }

  if (clippings1x || clippings2) {
    prefValue = commonClippings == true;
  }
  else {
    prefValue = true;
  }

  this._appUtils.setPref("clippings.datasource.common", prefValue);

  this._appUtils.log("aeClippings3._initCommonClippingsPref(): clippings.datasource.common = " + this._appUtils.getPref("clippings.datasource.common"));
};


/*
 * Process empty subfolders recursively and append the empty clipping
 * to each empty folder.
 * Prerequisite: Datasource must be already initialized.
 */
aeClippings3.prototype._processEmptyFolders = function ()
{
  try {
    this._clippingsSvc.processEmptyFolders();
  }
  catch (e) {
    throw this.E_CLIPPINGSSVC_NOT_INITIALIZED;
  }
};

